clear;
clc;
load('.\data\demo_data');
% data description:
% Gallery.X: gallery data (each cell is a set, whose columns represent single samples)
% Gallery.y: the label of gallery data
% Probe.X: probe data (each column is a sample)
% Probe.y: the label of proble data
% Note: the demo_data is our processed data by employing PCA
%%
%calculate mean and covariance matrix of each set data
[M_Gal,C_Gal] = Compute_Mean_Cov(Gallery.X);
[M_Pro,C_Pro] = Compute_Mean_Cov(Probe.X);

%calculate logarithm of covariance matrix
LogC_Gal = Compute_Log_Cov(C_Gal);
LogC_Pro = Compute_Log_Cov(C_Pro);

%calculate single Gaussian model of eath set data
SGM_Gal = Compute_SGM(Gallery.X);
SGM_Pro = Compute_SGM(Probe.X);

%calculate embedding model for each single Gaussian model 
G_Gal = Compute_SGM_Embedding(SGM_Gal);
G_Pro = Compute_SGM_Embedding(SGM_Pro);
%% calculate kernels for mean, covariance and Gaussian
K_Gal = cell(3,1);
K_Pro = cell(3,1);

%calculate linear kernel of mean model
K_Gal{1} = M_Gal'*M_Gal;
K_Pro{1} = M_Pro'*M_Gal;

%calculate linear kernel of covariance model
K_Gal{2} = Compute_Riemann_Kernel(LogC_Gal,[]);
K_Pro{2} = Compute_Riemann_Kernel(LogC_Pro,LogC_Gal);

%calculate linear kernel of single Gaussian model 
K_Gal{3} = Compute_Riemann_Kernel_Gau(G_Gal,[]);
K_Pro{3} = Compute_Riemann_Kernel_Gau(G_Pro,G_Gal);
%% kernel normalization
n_Gal = length(Gallery.y);
n_Pro = length(Probe.y);

n_Ker = 3; % the number of kernels
N_Gal=eye(n_Gal)-ones(n_Gal,n_Gal)/n_Gal;
N_Pro=ones(n_Pro,n_Gal)/n_Gal;
K_Gal_n = cell(n_Ker,1);
K_Pro_n = cell(n_Ker,1);

for i = 1 : n_Ker
    K_Pro_n{i} = (K_Pro{i} - N_Pro*K_Gal{i})*N_Gal;
    K_Gal_n{i} = N_Gal*K_Gal{i}*N_Gal;
end
%% Training for HERML model
if (~exist('params')),
    params = struct();
end

%parameter setting
params = SetDefaultParams(params);
params.gamma = 10; % common tuning range of [0.001 0.01 0.1 1 10 100 1000]
params.zeta = 0.1; % common tuning range of 0.1: 0.1: 1

%data constriant setting for metric learing
[upperbound, lowerbound, C, W] = GenConstrainMatrix(K_Gal_n,n_Ker,Gallery.y,params);

%initialization of HERML model
A_0 = cell(n_Ker,1);
for i = 1 : n_Ker
    A_0{i} = eye(size(K_Gal_n{i},2));
end

% training code of HERML model
A_herml = herml(C, K_Gal_n, A_0, n_Ker, params, W);

%calculate sqare root of HERML model
W_herml = cell(n_Ker,1);
for i = 1 : n_Ker
    W_temp = sqrtm(A_herml{i});
    tmpNorm = sqrt(sum((W_temp'* K_Gal{i}).*W_temp',2));
    W_temp = W_temp./repmat(tmpNorm',size(W_temp,1),1);
    W_herml{i} = W_temp;
end
%% calculate similarity
sim_mat = zeros(n_Gal,n_Pro);
for i = 1 : n_Ker
    Gallery_herml = K_Gal_n{i}*W_herml{i};
    Probe_herml = K_Pro_n{i}*W_herml{i};
    sim_mat = sim_mat+ (1- pdist2(Gallery_herml,Probe_herml,'cosine'));
end
%% calculate accuracy
sampleNum = length(Probe.y);
[sim ind] = sort(sim_mat,1,'descend');
correctNum = length(find((Probe.y-Gallery.y(ind(1,:)))==0));
fRate1 = correctNum/sampleNum;
fprintf('fRate = %f \n', fRate1);
